#! /bin/sh
 # This file (disk_cache_me.sh) was created by Ron Rechenmacher <ron@fnal.gov> on
 # Nov  1, 2016. "TERMS AND CONDITIONS" governing this file are in the README
 # or COPYING file. If you do not have such a file, one can be obtained by
 # contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
 # $RCSfile: disk_cache_me.sh,v $
 # rev='$Revision: 1.4 $$Date: 2016/11/15 22:16:10 $'

# determine system mem
# verify 2x disk space on local disk
# write files
# read files -- display rate


env_opts_var=`basename $0 | sed 's/\.sh$//' | tr 'a-z-' 'A-Z_'`_OPTS
USAGE="\
   usage: `basename $0` [-h?] [options] <test_dir>
examples: `basename $0` /tmp/xx
-f        force continue if test_dir not empty
--files=  specify num_files to write/read instead of system calculated.
--no-read  write only :)
--oflag=  flags for writer
--bigger  use 8 GB files instead of 1 GB
"
VUSAGE=

# Process script arguments and options
eval env_opts=\${$env_opts_var-} # can be args too
eval "set -- $env_opts \"\$@\""
op1chr='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set -- "-$rest" "$@"'
op1arg='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set --  "$rest" "$@"'
reqarg="$op1arg;"'test -z "${1+1}" &&echo opt -$op requires arg. &&echo "$USAGE" &&exit'
args= do_help= opt_v=0
while [ -n "${1-}" ];do
    if expr "x${1-}" : 'x-' >/dev/null;then
        op=`expr "x$1" : 'x-\(.*\)'`; shift   # done with $1
        leq=`expr "x$op" : 'x[^=]*\(=\)'` lev=`expr "x$op" : 'x[^=]*=\(.*\)'`
        test "x$leq" != x &&eval "set -- \"\$lev\" \"\$@\""&&op=`expr "x$op" : 'x\([^=]*\)'`
        case "$op" in
        \?*|h*)     eval $op1chr; do_help=1;;
        v*)         eval $op1chr; opt_v=`expr $opt_v + 1`;;
        x*)         eval $op1chr; test $opt_v -ge 1 && set -xv || set -x;;
        f*)         eval $op1chr; force=1;;
        -blksiz)    eval $reqarg; blksiz=$1;        shift;;
        -blkcnt)    eval $reqarg; blkcnt=$1;        shift;;
        blksiz)                   blksiz=$1;        shift;;
        blkcnt)                   blkcnt=$1;        shift;;
        -files)     eval $reqarg; num_files=$1;     shift;;
        -no-read)                 no_read=1;;
        -read-only)               only_read=1;;
        -oflag)     eval $reqarg; oflag=$1;     shift;;
        -bigger)                  MB=8192;;
        *)          echo "Unknown option -$op"; do_help=1;;
        esac
    else
        aa=`echo "$1" | sed -e"s/'/'\"'\"'/g"` args="$args '$aa'"; shift
    fi
done
eval "set -- $args \"\$@\""; unset args aa
help() { echo "$USAGE";test $opt_v -ge 1 && echo "$VUSAGE" || true for pipeline; }
test $# -ne 1 && do_help=1
test -n "${do_help-}" && help && exit
set -u

test_dir=$1

tdir=$test_dir
while [ ! -d $tdir -a $tdir != / ];do
    tdir=`dirname $tdir`
done

fs=`df $tdir | awk 'NR==2{print$1}'`
echo $fs | grep : >/dev/null && { echo testdir appears to be on nfs file system; exit 1; }

test -d $test_dir || mkdir -p $test_dir
cd $test_dir
files=`/bin/ls | wc -l`
echo initial number of files: $files
test $files -ne 0 -a -z "${force-}" && { echo test_dir must be empty; exit 1; }

sysmem_MB=`free -m | awk '/^Mem/{print$2;exit}'`
blksiz=${blksiz-32k}

# write 1 GB or 8 GB files
test -z "${MB-}" && MB=1024
if [ -z "${num_files-}" ];then
    # files enough for 2x system mem
    num_files=`expr $sysmem_MB \* 2 / $MB` || num_files=1
fi
echo num_files=$num_files

echo writing...
fsize=`expr $MB \* 1048576`
# create first 1 GB file
if [ ! -f test.dat.0 -o "`stat -c%s test.dat.0`" != $fsize ];then
    echo 123456789112345678921234567893123456789412345678951234567896123 >test.dat.0 # 64 bytes
    ls -l test.dat.0
    loops=`python -c "import math; print(\"%d\"%(math.log($fsize/64,2),))"`
    echo loops=$loops
    loops=`expr $loops + 1` # just to make while loops do the right number of loops
    while loops=`expr $loops - 1`;do
        cat test.dat.0 test.dat.0 >|test.dat.tmp
        mv test.dat.tmp test.dat.0
    done
fi

fidx=1
while [ $fidx -lt $num_files ];do
    if [ ! -f test.dat.$fidx -o "`stat -c%s test.dat.$fidx`" != $fsize ];then
        echo writing fidx=$fidx of num_files=$num_files
        dd if=test.dat.0 of=test.dat.$fidx bs=$blksiz ${oflag+oflag=$oflag} status=none
    fi
    fidx=`expr $fidx + 1`
done

reader()
{
    while true;do
        fidx=0
        while [ $fidx -lt $num_files ];do
            echo reading test.dat.$fidx
            wc=`dd if=test.dat.$fidx bs=$blksiz status=none | wc -l`
            fidx=`expr $fidx + 1`
        done
    done
}

other_writer()
{
    while true;do
        fidx=0
        while [ $fidx -lt $num_files ];do
            echo writing test.dat.$fidx.other
            dd if=test.dat.0 of=test.dat.$fidx.other bs=$blksiz ${oflag+oflag=$oflag} 2>&1 | grep -v records
            test $? -ne 0 && break 2
            fidx=`expr $fidx + 1`
        done
    done
}

cleanup() {
    echo "`basename $0` CLEANUP pids=${pids-}"
    if [ -n "${pids-}" ];then
        for pp in $pids;do
            for sig in INT HUP TERM;do
                if kill -s 0 $pp 2>/dev/null;then
                    echo `basename $0` killing $pp with $sig
                    kill -$sig $pp
                    sleep .1
                else
                    break
                fi
            done
        done
    fi
}


if [ -n "${only_read-}" ];then
    echo read only
    reader
elif [ -z "${no_read-}" ];then
    echo reading and writing
    trap cleanup EXIT
    trap exit TERM HUP
    reader &
    pids=$!

    other_writer
else
    echo write only
    other_writer
fi


cleanup
wait
sleep 1
