#! /bin/sh
 # This file (iperf_2D.sh) was created by Ron Rechenmacher <ron@fnal.gov> on
 # Nov  3, 2016. "TERMS AND CONDITIONS" governing this file are in the README
 # or COPYING file. If you do not have such a file, one can be obtained by
 # contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
 # $RCSfile: iperf_2D.sh,v $
 # rev='$Revision: 1.1 $$Date: 2016/11/15 22:17:43 $'

# Attempt simultaneous iperfs in both directions

#defaults
opt_time=20

USAGE="\
   usage: `basename $0` [-h?] [options] <remote_node>
examples: `basename $0` -t100 mu2edaq06-data
-t<seconds>  default=$opt_time
"
VUSAGE=

# Process script arguments and options
env_opts_var=`basename $0 | sed 's/\.sh$//' | tr 'a-z-' 'A-Z_'`_OPTS
eval env_opts=\${$env_opts_var-} # can be args too
eval "set -- $env_opts \"\$@\""
op1chr='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set -- "-$rest" "$@"'
op1arg='rest=`expr "$op" : "[^-]\(.*\)"`; test -n "$rest" && set --  "$rest" "$@"'
reqarg="$op1arg;"'test -z "${1+1}" &&echo opt -$op requires arg. &&echo "$USAGE" &&exit'
args= do_help= opt_v=0
while [ -n "${1-}" ];do
    if expr "x${1-}" : 'x-' >/dev/null;then
        op=`expr "x$1" : 'x-\(.*\)'`; shift   # done with $1
        leq=`expr "x$op" : 'x[^=]*\(=\)'` lev=`expr "x$op" : 'x[^=]*=\(.*\)'`
        test "x$leq" != x &&eval "set -- \"\$lev\" \"\$@\""&&op=`expr "x$op" : 'x\([^=]*\)'`
        case "$op" in
        \?*|h*)     eval $op1chr; do_help=1;;
        v*)         eval $op1chr; opt_v=`expr $opt_v + 1`;;
        x*)         eval $op1chr; test $opt_v -ge 1 && set -xv || set -x;;
        t*)         eval $op1arg; opt_time=$1; shift;;
        -time)      eval $reqarg; opt_time=$1; shift;;
        1*)         eval $op1chr; opt_1D=1;;
        o*)         eval $op1chr; ssh_o=-o$1;  shift;;
        *)          echo "Unknown option -$op"; do_help=1;;
        esac
    else
        aa=`echo "$1" | sed -e"s/'/'\"'\"'/g"` args="$args '$aa'"; shift
    fi
done
eval "set -- $args \"\$@\""; unset args aa
help() { echo "$USAGE";test $opt_v -ge 1 && echo "$VUSAGE" || true for pipeline; }
test $# -ne 1 && do_help=1
test -n "${do_help-}" && help && exit
set -u

IP2if()
{ # strange - some systems have the default route first -- need to sort these by metric to get them last???
  route -n | sed -n '/^[0-9]/p' | sort -rnk5 \
  |awk "function numit(ip){n=split(ip,a,\".\")+1;r=0;while(--n)r+=lshift(a[n],(4-n)*8);return r}
/^[0-9]/{if(and(numit(\$3),numit(\"$1\"))==numit(\$1)){print\$8;exit 0}}"
}

if2IP()    { /sbin/ifconfig $1 | sed -n '/inet addr:/{s/.*addr://;s/ .*//;p;q;}'; }

remote=$1
ping_out=`ping -c10 -i.2 -q $remote`
IP=`echo "$ping_out" | /bin/grep -m1 --only-matching '[0-9]*\.[0-9]*\.[0-9]*\.[0-9]*'`
IF=`IP2if $IP`
myIP=`if2IP $IF`
echo myIP=$myIP

cleanup() {
    echo "`basename $0` CLEANUP pids=${pids-}"
    #exec 2>/dev/null
    if [ -n "${pids-}" ];then
        for pp in $pids;do
            for sig in INT HUP TERM;do
                if kill -s 0 $pp 2>/dev/null;then
                    echo `basename $0` killing $pp with $sig
                    kill -$sig $pp
                    sleep .1
                else
                    break
                fi
            done
        done
    fi
}
trap cleanup EXIT
trap exit TERM HUP
iperf -s >/dev/null 2>&1 &
pids=$!
sleep 1

if [ -n "${opt_1D-}" ];then
    ssh $remote ${ssh_o-} "iperf -c$myIP -t$opt_time -i10"
else
    sleep=2
    ssh $remote ${ssh_o-} 'iperf -s >/dev/null 2>&1 & pid=$!;'"sleep $sleep;iperf -c$myIP -t$opt_time;"' kill $pid' &
    pids="$pids $!"
    sleep $sleep

    iperf -c$remote -t$opt_time -i30 &
    pids="$pids $!"
    wait
fi

sleep .5  # allow printing of any output from background processes.
